package com.example.citizen.citizen_androidsdk_sample2;

import com.citizen.sdk.labelprint.LabelConst;
import com.citizen.sdk.labelprint.LabelPrinter;
import com.citizen.sdk.labelprint.LabelDesign;

import android.app.Activity;
import android.widget.Button;
import android.widget.EditText;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Toast;
import android.app.AlertDialog;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.hardware.usb.UsbDevice;

import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;

public class MainActivity extends AppCompatActivity {
    final String PREFERENCES_NAME = "Citizen_Sample_2_for_LabelPrinterLibrary";
    final String PRE_KEY_IP_ADDRESS = "IP_Address";
    final String DEFAULT_IP_ADDRESS = "192.168.128.212";
    final String PRE_KEY_BLUETOOTH_ADDRESS = "Bluetooth_Address";
    final String DEFAULT_BLUETOOTH_ADDRESS = "00:01:90:E8:03:33";

    private LabelPrinter printer;

    @Override
    protected void onCreate( Bundle savedInstanceState ) {
        super.onCreate( savedInstanceState );
        setContentView( R.layout.activity_main );

        // Create an instance( LabelPrinter class )
        printer = new LabelPrinter();

        //
        // WiFi & USB address edit disable
        //
        final EditText editText_Address_Usb = (EditText) findViewById(R.id.editText_UsbDevice);
        if (editText_Address_Usb != null)
        {
            editText_Address_Usb.setKeyListener(null);
        }

        //
        // Load Address from Preferences
        //
        final EditText editText_Address_WiFi = (EditText) findViewById(R.id.editText_Address_WiFi);
        final EditText editText_Address_Bluetooth = (EditText) findViewById(R.id.editText_Address_Bluetooth);
        if (editText_Address_WiFi != null)
        {
            editText_Address_WiFi.setText(loadAddress(LabelConst.CLS_PORT_WiFi));
        }
        if (editText_Address_Bluetooth != null)
        {
            editText_Address_Bluetooth.setText(loadAddress(LabelConst.CLS_PORT_Bluetooth));
        }

        //
        // Connect Button - Wi-Fi
        //
        Button button_Connect_WiFi = (Button) findViewById(R.id.button_Connect_WiFi);
        if (button_Connect_WiFi != null)
        {
            button_Connect_WiFi.setOnClickListener(new OnClickListener() {
                @Override
                public void onClick(View v) {

                    // Get Address
                    String address = "";
                    if (editText_Address_WiFi != null) {
                        address = editText_Address_WiFi.getText().toString();
                    }

                    // Connect printer
                    connect(printer, LabelConst.CLS_PORT_WiFi, address);

                }
            });
        }

        //
        // Disconnect Button - Wi-Fi
        //
        Button button_Disconnect_WiFi = (Button) findViewById(R.id.button_Disconnect_WiFi);
        if (button_Disconnect_WiFi != null)
        {
            button_Disconnect_WiFi.setOnClickListener(new OnClickListener() {
                @Override
                public void onClick(View v) {

                    // Disconnect
                    disconnect(printer);

                }
            });
        }

        //
        // Sample Print Button - Wi-Fi
        //
        Button button_SamplePrint01_WiFi = (Button) findViewById(R.id.button_SamplePrint01_WiFi);
        if (button_SamplePrint01_WiFi != null)
        {
            button_SamplePrint01_WiFi.setOnClickListener(new OnClickListener() {
                @Override
                public void onClick(View v) {

                    // Create an instance( LabelDesign class )
                    LabelDesign design = new LabelDesign();

                    // Text
                    design.drawTextPtrFont("Sample Print", LabelConst.CLS_LOCALE_JP, LabelConst.CLS_PRT_FNT_TRIUMVIRATE_B, LabelConst.CLS_RT_NORMAL, 1, 1, LabelConst.CLS_PRT_FNT_SIZE_24, 20, 300);

                    // QRCode
                    design.drawQRCode("DrawQRCode", LabelConst.CLS_ENC_CDPG_US_ASCII, LabelConst.CLS_RT_NORMAL, 4, LabelConst.CLS_QRCODE_EC_LEVEL_H, 20, 220);

                    // Rect(fill)
                    design.fillRect(20, 150, 350, 40, LabelConst.CLS_SHADED_PTN_11);

                    // BarCode
                    design.drawBarCode("0123456789", LabelConst.CLS_BCS_CODE128, LabelConst.CLS_RT_NORMAL, 3, 3, 30, 20, 70, LabelConst.CLS_BCS_TEXT_SHOW);


                    int printDarkness = printer.getPrintDarkness();
                    if (LabelConst.CLS_PROPERTY_DEFAULT == printDarkness) {
                        // Set Properties
                        printer.setPrintDarkness(10);
                    }

                    // Print Label
                    int result = printer.print(design, 0001);
                    if (LabelConst.CLS_SUCCESS != result) {
                        // Print Error
                        Toast.makeText(MainActivity.this, "Print Error :" + Integer.toString(result), Toast.LENGTH_LONG).show();
                    }
                }
            });
        }

        //
        // Sample Status Button - Wi-Fi
        //
        Button button_SamplePrint02_WiFi = (Button) findViewById(R.id.button_SamplePrint02_WiFi);
        if (button_SamplePrint02_WiFi != null) {
            button_SamplePrint02_WiFi.setOnClickListener(new OnClickListener() {
                @Override
                public void onClick(View v) {

                    // Status
                    status(printer);

                }
            });
        }

        //
        // Connect Button - BlueTooth
        //
        Button button_Connect_BlueTooth = (Button) findViewById(R.id.button_Connect_Bluetooth);
        if (button_Connect_BlueTooth != null)
        {
            button_Connect_BlueTooth.setOnClickListener(new OnClickListener() {
                @Override
                public void onClick(View v) {

                    // Get Address
                    String address = "";
                    if (editText_Address_Bluetooth != null) {
                        address = editText_Address_Bluetooth.getText().toString();
                    }

                    // Connect printer
                    connect(printer, LabelConst.CLS_PORT_Bluetooth, address);               // Android 2.3.2 ( API Level 9 ) or before
                    // connect(printer, LabelConst.CLS_PORT_Bluetooth_Insecure, address);      // Android 2.3.3 ( API Level 10) or later

                }
            });
        }

        //
        // Disconnect Button - BlueTooth
        //
        Button button_Disconnect_BlueTooth = (Button) findViewById(R.id.button_DisConnect_Bluetooth);
        if (button_Disconnect_BlueTooth != null)
        {
            button_Disconnect_BlueTooth.setOnClickListener(new OnClickListener() {
                @Override
                public void onClick(View v) {

                    // Disconnect
                    disconnect(printer);

                }
            });
        }

        //
        // Sample Print Button - BlueTooth
        //
        Button button_SamplePrint01_Bluetooth = (Button) findViewById(R.id.button_SamplePrint01_Bluetooth);
        if (button_SamplePrint01_Bluetooth != null) {
            button_SamplePrint01_Bluetooth.setOnClickListener(new OnClickListener() {
                @Override
                public void onClick(View v) {

                    // Print data output
                    print(printer);

                }
            });
        }

        //
        // Sample Status Button - BlueTooth
        //
        Button button_SamplePrint02_Bluetooth = (Button) findViewById(R.id.button_SamplePrint02_Bluetooth);
        if (button_SamplePrint02_Bluetooth != null) {
            button_SamplePrint02_Bluetooth.setOnClickListener(new OnClickListener() {
                @Override
                public void onClick(View v) {

                    // Status
                    status(printer);

                }
            });
        }

        //
        // Connect Button - USB
        //
        Button button_Connect_Usb = (Button) findViewById(R.id.button_Connect_Usb);
        if (button_Connect_Usb != null)
        {
            button_Connect_Usb.setOnClickListener(new OnClickListener() {
                @Override
                public void onClick(View v) {

                    // Get Address
                    String address = "";
                    if (editText_Address_Usb != null) {
                        address = editText_Address_Usb.getText().toString();
                    }

                    // Connect printer
                    connect(printer, LabelConst.CLS_PORT_USB, address);

                }
            });
        }

        //
        // Disconnect Button - USB
        //
        Button button_Disconnect_Usb = (Button) findViewById(R.id.button_DisConnect_Usb);
        if (button_Disconnect_Usb != null)
        {
            button_Disconnect_Usb.setOnClickListener(new OnClickListener() {
                @Override
                public void onClick(View v) {

                    // Disconnect
                    disconnect(printer);

                }
            });
        }

        //
        // Sample Print Button - USB
        //
        Button button_SamplePrint01_USB = (Button) findViewById(R.id.button_SamplePrint01_USB);
        if (button_SamplePrint01_USB != null) {
            button_SamplePrint01_USB.setOnClickListener(new OnClickListener() {
                @Override
                public void onClick(View v) {

                    // Print data output
                    print(printer);

                }
            });
        }

        //
        // Sample Status Button - USB
        //
        Button button_SamplePrint02_USB = (Button) findViewById(R.id.button_SamplePrint02_USB);
        if (button_SamplePrint02_USB != null) {
            button_SamplePrint02_USB.setOnClickListener(new OnClickListener() {
                @Override
                public void onClick(View v) {

                    // Status
                    status(printer);

                }
            });
        }

    }

    //
    // print
    //
    private void print(LabelPrinter printer)
    {
        LabelDesign design = new LabelDesign();
        int errflg = 0;
        int result;
        String errmsg = "";

        // Text
        result = design.drawTextPtrFont("Sample Print", LabelConst.CLS_LOCALE_JP, LabelConst.CLS_PRT_FNT_TRIUMVIRATE_B, LabelConst.CLS_RT_NORMAL, 1, 1, LabelConst.CLS_PRT_FNT_SIZE_24, 20, 300);
        if (LabelConst.CLS_SUCCESS != result)
        {
            errmsg = errmsg + Integer.toString(result) + ":\tdrawTextPtrFont Error\r\n";
            errflg = 1;
        }

        // QRCode
        result = design.drawQRCode("DrawQRCode", LabelConst.CLS_ENC_CDPG_US_ASCII, LabelConst.CLS_RT_NORMAL, 4, LabelConst.CLS_QRCODE_EC_LEVEL_H, 20, 220);
        if (LabelConst.CLS_SUCCESS != result)
        {
            errmsg = errmsg + Integer.toString(result) + ":\tdrawQRCode Error\r\n";
            errflg = 1;
        }

        // Rect(fill)
        result = design.fillRect(20, 150, 350, 40, LabelConst.CLS_SHADED_PTN_11);
        if (LabelConst.CLS_SUCCESS != result)
        {
            errmsg = errmsg + Integer.toString(result) + ":\tfillRect Error\r\n";
            errflg = 1;
        }

        // BarCode
        result = design.drawBarCode("0123456789", LabelConst.CLS_BCS_CODE128, LabelConst.CLS_RT_NORMAL, 3, 3, 30, 20, 70, LabelConst.CLS_BCS_TEXT_SHOW );
        if (LabelConst.CLS_SUCCESS != result)
        {
            errmsg = errmsg + Integer.toString(result) + ":\tdrawBarCode Error\r\n";
            errflg = 1;
        }

        // Print
        result = printer.print(design, 1);
        if (LabelConst.CLS_SUCCESS != result)
        {
            errmsg = errmsg + Integer.toString(result) + ":\tprint Error";
            errflg = 1;
        }

        // Err Message
        if (errflg != 0)
        {
            Toast.makeText(MainActivity.this, errmsg, Toast.LENGTH_LONG).show();
        }

    }

    //
    // Connect
    //
    private void connect(LabelPrinter printer, int portType, String address)
    {
        // Connect printer
        int result = LabelConst.CLS_SUCCESS;

        if (LabelConst.CLS_PORT_USB == portType)
        {
            // Set context
            printer.setContext(MainActivity.this);

            UsbDevice usbDevice = null;                         // null (Automatic detection)
            result = printer.connect(portType, usbDevice);      // Android 3.1 ( API Level 12 ) or later
        }
        else
        {
            //WiFi or Bluetooth
            result = printer.connect(portType, address);
        }


        if (LabelConst.CLS_SUCCESS == result) {
            // Connect Success
            Toast.makeText(MainActivity.this, "Connect Success :" + Integer.toString(result), Toast.LENGTH_LONG).show();
        }
        else {
            // Connect Error
            Toast.makeText(MainActivity.this, "Connect or Printer Error :" + Integer.toString(result), Toast.LENGTH_LONG).show();
        }

        saveAddress(portType,  address);
    }

    //
    // Disonnect
    //
    private void disconnect(LabelPrinter printer)
    {
        // Disconnect printer
        int result = printer.disconnect();
        if (LabelConst.CLS_SUCCESS == result) {
            // Disconnect Success
            Toast.makeText(MainActivity.this, "Disconnect Success :" + Integer.toString(result), Toast.LENGTH_LONG).show();
        }
        else {
            // Disconnect Error
            Toast.makeText(MainActivity.this, "Disconnect Error :" + Integer.toString(result), Toast.LENGTH_LONG).show();
        }
    }

    //
    // Status
    //
    private void status(LabelPrinter printer)
    {
        String msg = "";
        int result;

        // Checks the status and populates into each property
        result = printer.printerCheck();
        if (LabelConst.CLS_SUCCESS != result)
        {
            // SendData Error
            alertDialog("Result", "Send Error : " + Integer.toString(result));
            return;
        }

        msg = msg + Integer.toString(printer.getCommandInterpreterInAction()) + ":\tCommand interpreter in action" + "\r\n";
        msg = msg + Integer.toString(printer.getPaperError()) + ":\tPaper error" + "\r\n";
        msg = msg + Integer.toString(printer.getRibbonEnd()) + ":\tRibbon end" + "\r\n";
        msg = msg + Integer.toString(printer.getBatchProcessing()) + ":\tBatch processing" + "\r\n";
        msg = msg + Integer.toString(printer.getPrinting()) + ":\tPrinting" + "\r\n";
        msg = msg + Integer.toString(printer.getPause()) + ":\tPause" + "\r\n";
        msg = msg + Integer.toString(printer.getWaitingForPeeling()) + ":\tWaiting for peeling" + "\r\n";

        alertDialog("Status data", msg);
    }

    //
    // Alert Dialog
    //
    private void alertDialog(String title, String message)
    {
        new AlertDialog.Builder(this)
        .setTitle(title)
        .setMessage(message)
        .setPositiveButton("OK", null)
        .show();
    }

    //
    // Save & Load Address
    //
    private void saveAddress(int portType, String address)
    {
        switch(portType)
        {
            case LabelConst.CLS_PORT_WiFi:
                savePreferences( PRE_KEY_IP_ADDRESS, address );
                break;
            case LabelConst.CLS_PORT_Bluetooth:
            case LabelConst.CLS_PORT_Bluetooth_Insecure:
                savePreferences(PRE_KEY_BLUETOOTH_ADDRESS, address);
                break;
            default:
                break;
        }
    }
    private String loadAddress(int portType)
    {
        String address = "";
        switch (portType)
        {
            case LabelConst.CLS_PORT_WiFi:
                address = loadPreferences( PRE_KEY_IP_ADDRESS, DEFAULT_IP_ADDRESS );
                break;
            case LabelConst.CLS_PORT_Bluetooth:
            case LabelConst.CLS_PORT_Bluetooth_Insecure:
                address = loadPreferences(PRE_KEY_BLUETOOTH_ADDRESS, DEFAULT_BLUETOOTH_ADDRESS);
                break;
            default:
                break;
        }
        return address;
    }
    private void savePreferences(String sKeyName, String sValue)
    {
        SharedPreferences sp = getSharedPreferences(PREFERENCES_NAME, Activity.MODE_PRIVATE);
        Editor e = sp.edit();
        e.putString(sKeyName, sValue);
        e.commit();
    }
    private String loadPreferences(String sKeyName, String sDefaultValue)
    {
        SharedPreferences sp = getSharedPreferences(PREFERENCES_NAME, Activity.MODE_PRIVATE);
        return sp.getString(sKeyName, sDefaultValue);
    }

}
