package citizen.example.com.labeldesigning;

import com.citizen.sdk.labelprint.LabelConst;
import com.citizen.sdk.labelprint.LabelPrinter;
import com.citizen.sdk.labelprint.LabelDesign;
import com.citizen.sdk.labelprint.CitizenPrinterInfo;


import android.app.AlertDialog;
import android.content.DialogInterface;
import android.graphics.Typeface;
import android.util.Log;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.EditText;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Spinner;
import android.widget.Toast;
import android.hardware.usb.UsbDevice;

import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;

import java.text.MessageFormat;

public class MainActivity extends AppCompatActivity {

    // Types of errors
    enum ERRORKIND
    {
        ERR_CONNECT,
        ERR_PRINTERCHECK,
        ERR_PRINT,
        ERR_PRINTERSTATUS
    }

    // Get line feed code
    static final String BR = System.getProperty("line.separator");

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        // Select interface
        Spinner spinner = (Spinner) findViewById(R.id.spinner);
        if (spinner != null) {
            spinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
                @Override
                public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                    String address = "";
                    int ifType;

                    ifType = getSelectedInterface();
                    if (LabelConst.CLS_PORT_USB == ifType) {
                        address = "null(Automatic Detection)";
                    }
                    setAddress(address);
                }

                @Override
                public void onNothingSelected(AdapterView<?> parent) {
                    Log.e("OnNothingSelected", "NothingSelected");
                }
            });
        }

        // Search
        Button button_Search = (Button) findViewById(R.id.button_search);
        if (button_Search != null) {
            button_Search.setOnClickListener(new OnClickListener() {
                @Override
                public void onClick(View v) {

                    int searchMode = 0;     // SearchMode == 0  :   SearchCitizenPrinter
                                            //            != 0  :   SearchLabelPrinter
                    int ifType;
                    int timeout;
                    int[] errCode = new int[1];

                    // Create an instance( LabelPrinter class )
                    LabelPrinter printer = new LabelPrinter();

                    // Get selected interface
                    ifType = getSelectedInterface();
                    if (ifType == LabelConst.CLS_PORT_USB) {
                        // Usb interface is Auto Detect only
                        Toast.makeText(MainActivity.this, "USB interface is Automatic Detection only", Toast.LENGTH_LONG).show();
                        return;
                    }

                    // Set context
                    printer.setContext(MainActivity.this);

                    // Get timeout
                    timeout = getTimeOut(ifType);

                    // Search
                    if (searchMode == 0) {
                        // Search Citizen Printer
                        CitizenPrinterInfo list1[] = printer.searchCitizenPrinter(ifType, timeout, errCode);

                        if (list1.length > 0) {

                            final CharSequence[] item = new CharSequence[list1.length];
                            for (int i = 0; i < list1.length; i++) {
                                if (ifType == LabelConst.CLS_PORT_WiFi) {
                                    item[i] = list1[i].ipAddress;
                                } else {
                                    item[i] = list1[i].bdAddress;
                                }
                            }

                            // Search address preview and select
                            new AlertDialog.Builder(MainActivity.this)
                                    .setTitle("Select the address")
                                    .setItems(item,
                                            new DialogInterface.OnClickListener() {
                                                @Override
                                                public void onClick(DialogInterface dialog, int which) {
                                                    setAddress(String.format("%s", item[which]));
                                                }
                                            })
                                    .show();

                        } else {
                            // 0 search results
                            Toast.makeText(MainActivity.this, "0 Search Results", Toast.LENGTH_LONG).show();
                        }
                    } else {
                        // Search Label Printer
                        String[] list2 = printer.searchLabelPrinter(ifType, timeout, errCode);

                        if (list2.length > 0) {

                            final CharSequence[] item = list2;

                            // Search address preview and select
                            new AlertDialog.Builder(MainActivity.this)
                                    .setTitle("Select the address")
                                    .setItems(item,
                                            new DialogInterface.OnClickListener() {
                                                @Override
                                                public void onClick(DialogInterface dialog, int which) {
                                                    setAddress(String.format("%s", item[which]));
                                                }
                                            })
                                    .show();

                        } else {
                            // 0 search results
                            Toast.makeText(MainActivity.this, "0 Search Results", Toast.LENGTH_LONG).show();
                        }

                    }
                }
            });
        }

        // Example1
        //  Label Print (Single)
        Button button_print = (Button) findViewById(R.id.button_printSingle);
        if (button_print != null) {
            button_print.setOnClickListener(new OnClickListener() {
                @Override
                public void onClick(View v) {

                    int ret = LabelConst.CLS_SUCCESS;

                    // Create an instance ( LabelPrinter class )
                    LabelPrinter printer = new LabelPrinter();

                    // Get selected interface
                    int type = getSelectedInterface();

                    // Get address
                    String address = getAddress();

                    // Connect
                    if (type == LabelConst.CLS_PORT_USB)
                    {
                        // USB
                        printer.setContext(MainActivity.this);
                        UsbDevice usbDevice = null;
                        ret = printer.connect(LabelConst.CLS_PORT_USB, usbDevice);
                    }
                    else
                    {
                        // WiFi, Bluetooth
                        ret = printer.connect(type, address);
                    }

                    // Check connect result
                    if (ret != LabelConst.CLS_SUCCESS)
                    {
                        // Connect Error
                        Toast.makeText(MainActivity.this, "Connect or Printer Error : " + Integer.toString(ret), Toast.LENGTH_LONG).show();
                        return;
                    }

                    // Set Property (Measurement unit)
                    printer.setMeasurementUnit(LabelConst.CLS_UNIT_INCH);

                    // Create an instance( LabelDesign class )
                    LabelDesign design = new LabelDesign();

                    // Design label
                    DesignLabel_Pattern1(design);

                    // Print label
                    ret = printer.print(design, 1);

                    // Check print result
                    if (ret != LabelConst.CLS_SUCCESS)
                    {
                        // Print Error
                        Toast.makeText(MainActivity.this, "Print Error : " + Integer.toString(ret), Toast.LENGTH_LONG).show();
                    }

                    // Disconnect
                    printer.disconnect();
                }
            });
        }

        // Example2
        //  Label Print (Single)
        Button button_printMulti = (Button) findViewById(R.id.button_printMulti);
        if (button_printMulti != null) {
            button_printMulti.setOnClickListener(new OnClickListener() {
                @Override
                public void onClick(View v) {

                    int quantity = 3;
                    int serialNo = 0;
                    int ret = LabelConst.CLS_SUCCESS;

                    // Create an instance ( LabelPrinter class )
                    LabelPrinter printer = new LabelPrinter();

                    // Get selected interface
                    int type = getSelectedInterface();

                    // Get address
                    String address = getAddress();

                    // Connect
                    if (type == LabelConst.CLS_PORT_USB)
                    {
                        // USB
                        printer.setContext(MainActivity.this);
                        UsbDevice usbDevice = null;
                        ret = printer.connect(LabelConst.CLS_PORT_USB, usbDevice);
                    }
                    else
                    {
                        // WiFi, Bluetooth
                        ret = printer.connect(type, address);
                    }

                    // Check connect result
                    if (ret != LabelConst.CLS_SUCCESS)
                    {
                        // Connect Error
                        Toast.makeText(MainActivity.this, "Connect or Printer Error : " + Integer.toString(ret), Toast.LENGTH_LONG).show();
                        return;
                    }

                    // Set Property (Measurement unit)
                    printer.setMeasurementUnit(LabelConst.CLS_UNIT_INCH);

                    //---- Print Design2 ----//
                    for (int loopCnt = 1; loopCnt <= quantity; loopCnt++)
                    {
                        // Create an instance (LabelDesign Class)
                        LabelDesign design = new LabelDesign();

                        // Design label
                        DesignLabel_Pattern2(design, serialNo);

                        if (loopCnt < quantity)
                        {
                            // Set Property (Tear Off)
                            printer.setMediaHandling(LabelConst.CLS_MEDIAHANDLING_TEAROFF);

                            // Print design
                            printer.print(design, 1);
                        }
                        else
                        {
                            // Set Property (Cut on last page)
                            printer.setMediaHandling(LabelConst.CLS_MEDIAHANDLING_CUT);

                            // Print label
                            printer.print(design, 1);
                        }

                        // Count up serial number
                        serialNo++;
                    }

                    // Disconnect
                    printer.disconnect();
                }
            });
        }

        // Example3
        //  Print and check the printer status
        Button button_printStsChk = (Button) findViewById(R.id.button_printStsChk);
        if (button_printStsChk != null) {
            button_printStsChk.setOnClickListener(new OnClickListener() {
                @Override
                public void onClick(View v) {

                    int ret = LabelConst.CLS_SUCCESS;
                    int[] status = new int[1];
                    String msg = "";

                    // Create an instance (LabelPrinter Class)
                    LabelPrinter printer = new LabelPrinter();

                    ret = example3_connect(printer);
                    if (ret != LabelConst.CLS_SUCCESS)
                    {
                        // Error display and disconnect
                        example3_printerErrorProc(printer, ERRORKIND.ERR_CONNECT, ret, null);
                        return;
                    }

                    ret = example3_getPrinterStatus(printer);
                    if (ret != LabelConst.CLS_SUCCESS)
                    {
                        // Error display and disconnect
                        example3_printerErrorProc(printer, ERRORKIND.ERR_PRINTERCHECK, ret, null);
                        return ;
                    }

                    msg = example3_checkPrinterStatus(printer, status);
                    if (status[0] != 0)
                    {
                        // Error display and disconnect
                        example3_printerErrorProc(printer, ERRORKIND.ERR_PRINTERSTATUS, ret, msg);
                        return ;
                    }

                    ret = example3_print(printer);
                    if (ret != LabelConst.CLS_SUCCESS)
                    {
                        // Error display and disconnect
                        example3_printerErrorProc(printer, ERRORKIND.ERR_PRINT, ret, null);
                        return ;
                    }

                    msg = example3_checkPrinterStatus(printer, status);
                    if (status[0] != 0)
                    {
                        // Error display and disconnect
                        example3_printerErrorProc(printer, ERRORKIND.ERR_PRINTERSTATUS, 0, msg);
                        return ;
                    }
                    else
                    {
                        Toast.makeText(MainActivity.this, "Print Result : Success", Toast.LENGTH_LONG).show();
                    }

                    printer.disconnect();
                }
            });
        }

    }

    private void DesignLabel_Pattern1(LabelDesign design)
    {
        // Text
        mediumFont(design, "12 LBS     1 OF 2", 230, 560);

        smallFont(design, "CITIZEN SYSTEMS AMERICA", 10, 565);
        smallFont(design, "1234567890", 10, 555);
        smallFont(design, "STE 404", 10, 545);
        smallFont(design, "363 VAN NESS WAY", 10, 535);
        smallFont(design, "TORRANCE  CA 90501", 10, 525);

        mediumBoldFont(design, "SHIP", 15, 500);
        mediumBoldFont(design, "TO:", 19, 472);

        mediumFont(design, "LYMAN C SMITH", 60, 500);
        mediumFont(design, "9876543210", 60, 482);
        mediumFont(design, "SMITH TOWER", 60, 464);
        mediumFont(design, "STE 220", 60, 446);
        mediumFont(design, "506 2ND AVE", 60, 428);

        mediumBoldFont(design, "SEATTLE WA 98104-2307", 60, 407);

        // Line
        design.drawLine(0, 405, 450, 405, 1);
        design.drawLine(122, 404, 122, 290, 1);
        design.drawLine(0, 284, 450, 284, 10);

        // MaxiCode
        String[] maxicode = { "98104", "2307", "840", "001", "1Z123X560120754868" };
        design.drawMaxiCode(maxicode, LabelConst.CLS_RT_NORMAL, 10, 298);

        // BarCode
        design.drawBarCode("&D420981042307", LabelConst.CLS_BCS_CODE128, LabelConst.CLS_RT_NORMAL, 6, 3, 50, 180, 320, LabelConst.CLS_BCS_TEXT_HIDE);
        mediumFont(design, "(420) 98104-2307", 200, 295);

        smallFont(design, "(420) SHIP TO POSTAL CODE", 140, 385);

        design.drawTextLocalFont("UPS NEXT DAY AIR", Typeface.SERIF, LabelConst.CLS_RT_NORMAL, 100, 100, 20, LabelConst.CLS_FNT_BOLD, 10, 245);
        mediumSmallFont(design, "TRACKING #: 1Z 123 X56 01 2075 4868", 10, 230);
        design.drawTextLocalFont("1", Typeface.SERIF, LabelConst.CLS_RT_NORMAL, 100, 100, 42, LabelConst.CLS_FNT_BOLD, 310, 215);

        design.drawLine(0, 225, 450, 225, 1);
        design.drawBarCode("1Z123X560120754868", LabelConst.CLS_BCS_CODE128, LabelConst.CLS_RT_NORMAL, 6, 3, 100, 30, 105, LabelConst.CLS_BCS_TEXT_HIDE);
        design.drawLine(0, 84, 450, 84, 10);
    }

    private void DesignLabel_Pattern2(LabelDesign design, int serialNo)
    {
        design.drawTextPtrFont("Product Name: A1B23CD456", LabelConst.CLS_LOCALE_JP, LabelConst.CLS_PRT_FNT_TRIUMVIRATE, LabelConst.CLS_RT_NORMAL, 1, 1, LabelConst.CLS_PRT_FNT_SIZE_14, 10, 215);
        design.drawBarCode("A1B23CD456", LabelConst.CLS_BCS_CODE128, LabelConst.CLS_RT_NORMAL, 6, 3, 60, 10, 10, LabelConst.CLS_BCS_TEXT_HIDE);

        String tmpMsg = "Serial No: ";
        tmpMsg += Integer.toString(serialNo);
        design.drawTextPtrFont(tmpMsg, LabelConst.CLS_LOCALE_JP, LabelConst.CLS_PRT_FNT_TRIUMVIRATE, LabelConst.CLS_RT_NORMAL, 1, 1, LabelConst.CLS_PRT_FNT_SIZE_14, 10, 75);
        design.drawBarCode(tmpMsg, LabelConst.CLS_BCS_CODE128, LabelConst.CLS_RT_NORMAL, 6, 3, 60, 10, 150, LabelConst.CLS_BCS_TEXT_HIDE);
    }

    private void DesignLabel_Pattern3(LabelDesign design)
    {
        // Order No
        design.drawTextPtrFont("Order No:", LabelConst.CLS_LOCALE_JP, LabelConst.CLS_PRT_FNT_TRIUMVIRATE, LabelConst.CLS_RT_RIGHT90, 1, 1, LabelConst.CLS_PRT_FNT_SIZE_14, 360, 280);

        // Date
        design.drawTextPtrFont("Date:", LabelConst.CLS_LOCALE_JP, LabelConst.CLS_PRT_FNT_TRIUMVIRATE, LabelConst.CLS_RT_RIGHT90, 1, 1, LabelConst.CLS_PRT_FNT_SIZE_14, 320, 280);

        // To:
        design.drawTextPtrFont("To:", LabelConst.CLS_LOCALE_JP, LabelConst.CLS_PRT_FNT_TRIUMVIRATE, LabelConst.CLS_RT_RIGHT90, 1, 1, LabelConst.CLS_PRT_FNT_SIZE_14, 280, 280);

        // From:
        design.drawTextPtrFont("From:", LabelConst.CLS_LOCALE_JP, LabelConst.CLS_PRT_FNT_TRIUMVIRATE, LabelConst.CLS_RT_RIGHT90, 1, 1, LabelConst.CLS_PRT_FNT_SIZE_14, 240, 280);

        // Rectangle
        design.drawRect(230, 5, 155, 280, 1);

        // Order No
        design.drawTextPtrFont("Order No:", LabelConst.CLS_LOCALE_JP, LabelConst.CLS_PRT_FNT_TRIUMVIRATE, LabelConst.CLS_RT_RIGHT90, 1, 1, LabelConst.CLS_PRT_FNT_SIZE_14, 140, 280);

        // Date
        design.drawTextPtrFont("Date:", LabelConst.CLS_LOCALE_JP, LabelConst.CLS_PRT_FNT_TRIUMVIRATE, LabelConst.CLS_RT_RIGHT90, 1, 1, LabelConst.CLS_PRT_FNT_SIZE_14, 100, 280);

        // To:
        design.drawTextPtrFont("To:", LabelConst.CLS_LOCALE_JP, LabelConst.CLS_PRT_FNT_TRIUMVIRATE, LabelConst.CLS_RT_RIGHT90, 1, 1, LabelConst.CLS_PRT_FNT_SIZE_14, 60, 280);

        // From:
        design.drawTextPtrFont("From:", LabelConst.CLS_LOCALE_JP, LabelConst.CLS_PRT_FNT_TRIUMVIRATE, LabelConst.CLS_RT_RIGHT90, 1, 1, LabelConst.CLS_PRT_FNT_SIZE_14, 20, 280);

        // Rectangle
        design.drawRect(10, 5, 155, 280, 1);
    }

    private void smallFont(LabelDesign design, String data, int x, int y)
    {
        design.drawTextPtrFont(data,                    //string data
                LabelConst.CLS_LOCALE_OTHER,         //int locale
                LabelConst.CLS_PRT_FNT_TRIUMVIRATE, //int font
                LabelConst.CLS_RT_NORMAL,             //int rotation
                1,                                      //int hexp
                1,                                      //int vexp
                LabelConst.CLS_PRT_FNT_SIZE_8,       //int size
                x,                                      //int x
                y);                                     //int y
    }

    private void mediumSmallFont(LabelDesign design, String data, int x, int y)
    {
        design.drawTextPtrFont(data,                    //string data
                LabelConst.CLS_LOCALE_OTHER,         //int locale
                LabelConst.CLS_PRT_FNT_TRIUMVIRATE, //int font
                LabelConst.CLS_RT_NORMAL,            //int rotation
                1,                                     //int hexp
                1,                                     //int vexp
                LabelConst.CLS_PRT_FNT_SIZE_10,     //int size
                x,                                     //int x
                y);                                    //int y
    }

    private void mediumFont(LabelDesign design, String data, int x, int y)
    {
        design.drawTextPtrFont(data,                    //string data
                LabelConst.CLS_LOCALE_OTHER,         //int locale
                LabelConst.CLS_PRT_FNT_TRIUMVIRATE, //int font
                LabelConst.CLS_RT_NORMAL,            //int rotation
                1,                                     //int hexp
                1,                                     //int vexp
                LabelConst.CLS_PRT_FNT_SIZE_12,     //int size
                x,                                     //int x
                y);                                    //int y
    }

    private void mediumBoldFont(LabelDesign design, String data, int x, int y)
    {
        design.drawTextPtrFont(data,                         //string data
                LabelConst.CLS_LOCALE_OTHER,              //int locale
                LabelConst.CLS_PRT_FNT_TRIUMVIRATE_B,   //int font
                LabelConst.CLS_RT_NORMAL,                 //int rotation
                1,                                          //int hexp
                1,                                          //int vexp
                LabelConst.CLS_PRT_FNT_SIZE_14,          //int size
                x,                                          //int x
                y);                                         //int y
    }

    private int getTimeOut(int type)
    {
        int timeout;

        if (LabelConst.CLS_PORT_WiFi == type)
        {
            timeout = 3;
        }
        else if ((LabelConst.CLS_PORT_Bluetooth == type) || (LabelConst.CLS_PORT_Bluetooth_Insecure == type))
        {
            timeout = 10;
        }
        else
        {
            timeout = 0;
        }

        return timeout;
    }

    private int getSelectedInterface()
    {
        int selectIF = LabelConst.CLS_PORT_WiFi;
        Spinner spinner = (Spinner) findViewById(R.id.spinner);

        if (spinner != null) {
            selectIF = spinner.getSelectedItemPosition();
        }
        return selectIF;
    }

    private String getAddress()
    {
        String address = "";
        EditText editText_Address = (EditText) findViewById(R.id.editText_address);

        if (editText_Address != null)
        {
            address = editText_Address.getText().toString();
        }
        return address;
    }

    private void setAddress(String address)
    {
        EditText editText_Address = (EditText) findViewById(R.id.editText_address);

        if (editText_Address != null) {
            editText_Address.setText(address);
        }
    }

    private int example3_connect(LabelPrinter printer)
    {
        int ret = LabelConst.CLS_SUCCESS;

        // Get selected interface
        int type = getSelectedInterface();

        // Get address
        String address = getAddress();

        // Connect
        if (type == LabelConst.CLS_PORT_USB)
        {
            // USB
            printer.setContext(MainActivity.this);
            UsbDevice usbDevice = null;
            ret = printer.connect(LabelConst.CLS_PORT_USB, usbDevice);
        }
        else
        {
            // WiFi, Bluetooth
            ret = printer.connect(type, address);
        }

        return ret;
    }

    private int example3_getPrinterStatus(LabelPrinter printer)
    {
        int ret = LabelConst.CLS_SUCCESS;

        // Get the printer status
        ret = printer.printerCheck();

        return ret;
    }

    private String example3_checkPrinterStatus(LabelPrinter printer,int[] ret)
    {
        String retMsg;
        StringBuilder tmpMsg = new StringBuilder("");
        int status;

        // Check printer status
        {
            // Make message
            tmpMsg.append(MessageFormat.format("Printer Status{0}", BR));

            status = printer.getCommandInterpreterInAction();
            tmpMsg.append(MessageFormat.format(" {0}: Command interpreter in action{1}", status, BR));
            ret[0] += status;

            status = printer.getPaperError();
            tmpMsg.append(MessageFormat.format(" {0}: Paper error{1}", status, BR));
            ret[0] += status;

            status = printer.getRibbonEnd();
            tmpMsg.append(MessageFormat.format(" {0}: Ribbon end{1}", status, BR));
            ret[0] += status;

            status = printer.getBatchProcessing();
            tmpMsg.append(MessageFormat.format(" {0}: Batch processing{1}", status, BR));
            ret[0] += status;

            status = printer.getPrinting();
            tmpMsg.append(MessageFormat.format(" {0}: Printing{1}", status, BR));
            ret[0] += status;

            status = printer.getPause();
            tmpMsg.append(MessageFormat.format(" {0}: Pause{1}", status, BR));
            ret[0] += status;

            status = printer.getWaitingForPeeling();
            tmpMsg.append(MessageFormat.format(" {0}: Waiting for peeling{1}", status, BR));
            ret[0] += status;
        }

        retMsg = tmpMsg.toString();
        return retMsg;
    }

    private int example3_print(LabelPrinter printer)
    {
        int ret = LabelConst.CLS_SUCCESS;

        // Set Property (Measurement unit)
        printer.setMeasurementUnit(LabelConst.CLS_UNIT_INCH);

        // Create an instance (LabelDesign Class)
        LabelDesign design = new LabelDesign();

        // Design label
        DesignLabel_Pattern3(design);

        // Print label
        ret = printer.print(design, 2);
        if (ret != LabelConst.CLS_SUCCESS)
        {
            return ret;
        }

        try {
            Thread.sleep(500);
        } catch (InterruptedException e) {}

        // Check printing
        while (true) {
            // Get the printer status
            ret = printer.printerCheck();
            if (ret != LabelConst.CLS_SUCCESS) {
                return ret;
            }

            // Checks the Printing and BatchProcessing populates
            if ((printer.getPrinting() == 0) && (printer.getBatchProcessing() == 0)) {
                break;
            }
        }

        return ret;
    }

    private void example3_printerErrorProc(final LabelPrinter printer, ERRORKIND errKind, int errCode, String msg)
    {
        StringBuilder tmpMsg = new StringBuilder("");

        if (errKind == ERRORKIND.ERR_CONNECT)
        {
            Toast.makeText(MainActivity.this, "Connect failure: " + errCode, Toast.LENGTH_LONG).show();
        }
        else if (errKind == ERRORKIND.ERR_PRINTERCHECK)
        {
            Toast.makeText(MainActivity.this, "PrinterCheck failure: " + errCode, Toast.LENGTH_LONG).show();
            printer.disconnect();
        }
        else if (errKind == ERRORKIND.ERR_PRINT)
        {
            Toast.makeText(MainActivity.this, "Print failure: " + errCode, Toast.LENGTH_LONG).show();
            printer.disconnect();
        }
        else if (errKind == ERRORKIND.ERR_PRINTERSTATUS)
        {
            // Show error dialog
            AlertDialog.Builder alert = new AlertDialog.Builder(this);

            alert.setTitle("Printer Error");

            tmpMsg.append(MessageFormat.format("The printer can not print.{0}", BR));
            tmpMsg.append(MessageFormat.format("Select \"Yes\" to reset the printer, or \"No\" if you want to restore it manually.{0}", BR));
            tmpMsg.append(MessageFormat.format("*Clear print buffer is select \"Yes\".{0}{1}", BR, BR));
            tmpMsg.append(msg);
            alert.setMessage(tmpMsg);
            alert.setPositiveButton(android.R.string.yes, new DialogInterface.OnClickListener(){
                public void onClick(DialogInterface dialog, int which) {
                    printer.clearOutput();
                    try {
                        Thread.sleep(500);
                    } catch (InterruptedException e) {}

                    printer.disconnect();
                }
            });
            alert.setNegativeButton(android.R.string.no, new DialogInterface.OnClickListener(){
                public void onClick(DialogInterface dialog, int which) {
                    printer.disconnect();
                }
            });
            alert.setOnCancelListener(new DialogInterface.OnCancelListener() {
                public void onCancel(DialogInterface dialog) {
                    printer.disconnect();
                }
            });
            alert.show();
        }
        else
        {
            // No Operation
        }
    }

}
